@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2016 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -56,6 +56,18 @@
 
 #include "property.h"
 
+#ifndef WIN32
+ /*
+  * Mac OSX 10.5 does not define the environ variable.  This is work around for that.
+  */
+ #ifdef MACOSX
+  #include <crt_externs.h>
+  #define environ *_NSGetEnviron();
+ #endif
+
+extern char** environ;
+#endif
+
 /* The following define will enable debug output of the code to parse the JVM output. */
 /*#define DEBUG_CHILD_OUTPUT*/
 
@@ -78,9 +90,13 @@
 #define WRAPPER_TIMER_FAST_THRESHOLD (2 * 24 * 3600 * 1000 / WRAPPER_TICK_MS) /* Default to 2 days. */
 #define WRAPPER_TIMER_SLOW_THRESHOLD (2 * 24 * 3600 * 1000 / WRAPPER_TICK_MS) /* Default to 2 days. */
 
-#define WRAPPER_BACKEND_TYPE_UNKNOWN 0 /* Unknown type. */
-#define WRAPPER_BACKEND_TYPE_SOCKET  1 /* Use a loopback socket to communicate. */
-#define WRAPPER_BACKEND_TYPE_PIPE    2 /* Use a pair of pipes to communicate. */
+#define WRAPPER_BACKEND_ERROR_NEXT     99 /* value returned when there is an error trying to connect to a socket and we should try the next way to connect */
+#define WRAPPER_BACKEND_TYPE_UNKNOWN   0 /* Unknown type. */
+#define WRAPPER_BACKEND_TYPE_SOCKET_V4 0x01
+#define WRAPPER_BACKEND_TYPE_SOCKET_V6 0x02
+#define WRAPPER_BACKEND_TYPE_SOCKET    (WRAPPER_BACKEND_TYPE_SOCKET_V4 | WRAPPER_BACKEND_TYPE_SOCKET_V6)
+#define WRAPPER_BACKEND_TYPE_PIPE      0x04 /* Use a pair of pipes to communicate. */
+#define WRAPPER_BACKEND_TYPE_AUTO      (WRAPPER_BACKEND_TYPE_SOCKET | WRAPPER_BACKEND_TYPE_PIPE)
 
 #define WRAPPER_WSTATE_STARTING  51 /* Wrapper is starting.  Remains in this state
                                      *  until the JVM enters the STARTED state or
@@ -185,13 +201,6 @@ typedef unsigned int TICKS;
 typedef unsigned long TICKS;
 #endif
 
-#define WRAPPER_ENV_SOURCE_PARENT      1
-#define WRAPPER_ENV_SOURCE_WRAPPER     2
-#ifdef WIN32
-#define WRAPPER_ENV_SOURCE_REG_SYSTEM  4
-#define WRAPPER_ENV_SOURCE_REG_ACCOUNT 8
-#endif
-
 #ifdef WIN32
 /* Defines the maximum number of service manager control events that can be queued in a single loop. */
 #define CTRL_CODE_QUEUE_SIZE 26 /* Can enqueue one less than this count at any time. */
@@ -254,6 +263,8 @@ struct WrapperConfig {
     int     commandLogLevel;        /* The log level to use when logging the java command. */
     int     printJVMVersion;        /* tells the Wrapper to create a temp JVM to query the version, before starting the java application */
 #ifdef WIN32
+    int     argc;                   /* argc params as originally received in the entry function */
+    TCHAR   **argv;                 /* argv params as originally received in the entry function */
     TCHAR   *jvmVersionCommand;     /* Command used to launch the JVM and request its version */
     TCHAR   *jvmCommand;            /* Command used to launch the JVM */
 #else /* UNIX */
@@ -457,6 +468,7 @@ struct WrapperConfig {
 #endif
 };
 
+
 #define WRAPPER_SIGNAL_MODE_IGNORE   (char)100
 #define WRAPPER_SIGNAL_MODE_RESTART  (char)101
 #define WRAPPER_SIGNAL_MODE_SHUTDOWN (char)102
@@ -487,6 +499,9 @@ struct WrapperConfig {
 #define WRAPPER_PROCESS_DOWN      200
 #define WRAPPER_PROCESS_UP        201
 
+/* default timeout (in second) when executing child process to print java version */
+#define DEFAULT_JAVA_VERSION_TIMEOUT    10
+
 extern WrapperConfig *wrapperData;
 extern Properties    *properties;
 
@@ -533,6 +548,24 @@ extern int wrapperProtocolRead();
 /******************************************************************************
  * Utility Functions
  *****************************************************************************/
+
+/**
+ * Does any necessary post processing on the command string.
+ *  This function assumes that command has been malloced.  It will either return
+ *  the string as is, or return a modified string.  When a modified string is
+ *  returned the orignal command buffer will always be freed.
+ *
+ * 1) Replace the first instance of the %WRAPPER_COMMAND_FILLER_N% environment
+ *    variable so that the total command length will be equal to or greater than
+ *    the length specified by N.  The padding will be the length plus a series of
+ *    Xs terminated by a single Y.  This is mainly for testing.
+ *
+ * @param command The original command.
+ *
+ * @return The modifed command.
+ */
+extern TCHAR *wrapperPostProcessCommandElement(TCHAR *command);
+
 /**
  * Test function to pause the current thread for the specified amount of time.
  *  This is used to test how the rest of the Wrapper behaves when a particular
@@ -568,12 +601,41 @@ extern int wrapperWildcardMatch(const TCHAR *text, const TCHAR *pattern, size_t
  */
 extern size_t wrapperGetMinimumTextLengthForPattern(const TCHAR *pattern);
 
+/**
+ * Function that copies an array of strings.
+ *  The output will be pointer to a NULL terminated array even though the source does not end with a NULL value.
+ *
+ * @param arrOut Array with copied strings
+ * @param arrIn  Array to copy
+ * @param count  Number of elements to copy. If arrIn has less elements than count, arrOut will be the same size as arrIn.
+ */
+extern void wrapperCopyStringArray(TCHAR*** arrOut, TCHAR*** arrIn, int count);
+
+/**
+ * Function that splits a text into tokens according to delimiters.
+ *  Any spaces around delimiters will be trimmed.
+ *  Several delimiters following each others without spaces will be considered as one.
+ *  If there are any space between 2 delimiters, the function will consider it as a token and trim it.
+ *
+ * @param text Text to be split
+ * @param delim Array of delimiters 
+ *
+ * @return tokens in an Array of TCHAR*
+ */
+/*extern TCHAR** wrapperSplitText(const TCHAR* text, const TCHAR* delim);*/
+
+/**
+ * Trims any whitespace from the beginning and end of the in string
+ *  and places the results in the out buffer.  Assumes that the out
+ *  buffer is at least as large as the in buffer. */
+extern void trim(const TCHAR *in, TCHAR *out);
+
 /**
  * Returns a constant text representation of the specified Wrapper State.
  *
  * @param wState The Wrapper State whose name is being requested.
  *
- * @return Thre requested Wrapper State.
+ * @return The requested Wrapper State.
  */
 extern const TCHAR *wrapperGetWState(int wState);
 
@@ -582,7 +644,7 @@ extern const TCHAR *wrapperGetWState(int wState);
  *
  * @param jState The Java State whose name is being requested.
  *
- * @return Thre requested Java State.
+ * @return The requested Java State.
  */
 extern const TCHAR *wrapperGetJState(int jState);
 
@@ -736,6 +798,14 @@ extern void wrapperSetJavaState(int jState, TICKS nowTicks, int delay);
  * Platform specific methods
  *****************************************************************************/
 #ifdef WIN32
+/**
+ * Allocates an hidden console. (fix for the console flicker bug).
+ *  The size of the console will be minimized and its position will be set outside of the screen.
+ *  
+ * @return TRUE if the console was allocated, FALSE if it could not be allocated.
+ */
+extern int wrapperAllocHiddenConsole();
+
 extern void wrapperCheckConsoleWindows();
 /**
  *   checks the digital Signature of the binary and reports the result.
@@ -748,8 +818,13 @@ BOOL extern duplicateSTD();
 BOOL extern myShellExec(HWND hwnd, LPCTSTR pszVerb, LPCTSTR pszPath, LPCTSTR pszParameters, LPCTSTR pszDirectory, TCHAR* namedPipeName);
 BOOL extern runElevated( __in LPCTSTR pszPath, __in_opt LPCTSTR pszParameters, __in_opt LPCTSTR pszDirectory, TCHAR* namedPipeName);
 BOOL extern isElevated();
+BOOL extern isWin10OrHigher();
 BOOL extern isVista();
+BOOL extern isWinXP();
 extern void wrapperMaintainControlCodes();
+int wrapperServiceStatus(int consoleOutput);
+#define OSBUFSIZE 256
+BOOL GetOSDisplayString(TCHAR** pszOS);
 #else
 extern void wrapperMaintainSignals();
 extern TCHAR* findPathOf(const TCHAR *exe, const TCHAR* name);
@@ -805,8 +880,10 @@ extern void wrapperPauseBeforeExecute();
 
 /**
  * Launches a JVM process and store it internally
+ *
+ * @return TRUE if there were any problems.  When this happens the Wrapper will not try to restart.
  */
-extern void wrapperExecute();
+extern int wrapperExecute();
 
 /**
  * Returns a tick count that can be used in combination with the
@@ -894,6 +971,13 @@ extern void wrapperDetachJava();
  */
 extern void wrapperStopProcess(int exitCode, int force);
 
+/**
+ * Depending on the current state, we want to change the exact message displayed when restarting the JVM.
+ *
+ * The logic here needs to match that in wrapperRestartProcess.
+ */
+extern const TCHAR *wrapperGetRestartProcessMessage();
+
 /**
  * Used to ask the state engine to shut down the JVM.
  */
