@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2016 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -35,6 +35,10 @@
 
 #ifndef WIN32
 
+#ifdef LINUX
+ #include <features.h>
+#endif
+#include <limits.h>
 #include <wchar.h>
 #include <netdb.h>
 #include <stdio.h>
@@ -85,6 +89,12 @@ int pipedes[2] = {-1, -1};
 #define PIPE_READ_END 0
 #define PIPE_WRITE_END 1
 
+/**
+ * maximum length for a user name should be 8, 
+ * but according to 'man useradd' it may be 32
+ */
+#define MAX_USER_NAME_LENGTH 32
+
 TCHAR wrapperClasspathSeparator = TEXT(':');
 
 int javaIOThreadSet = FALSE;
@@ -215,7 +225,7 @@ void takeSignalAction(int sigNum, const TCHAR *sigName, int mode) {
         switch (mode) {
         case WRAPPER_SIGNAL_MODE_RESTART:
             log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_STATUS,
-                TEXT("%s trapped.  Restarting JVM."), sigName);
+                TEXT("%s trapped.  %s"), sigName, wrapperGetRestartProcessMessage());
             wrapperRestartProcess();
             break;
 
@@ -425,7 +435,7 @@ void descSignal(siginfo_t *sigInfo) {
  #ifdef UNICODE
     size_t req;
  #endif
-    TCHAR *uName;
+    TCHAR uName[MAX_USER_NAME_LENGTH + 1];
 #endif
 
     /* Not supported on all platforms */
@@ -459,24 +469,22 @@ void descSignal(siginfo_t *sigInfo) {
                 sigInfo->si_errno,
                 strerror(sigInfo->si_errno));
         }
-
+            
 #ifdef SI_USER
         if (sigInfo->si_code == SI_USER) {
             pw = getpwuid(sigInfo->si_uid);
             if (pw == NULL) {
-                uName = TEXT("<unknown>");
+                _sntprintf(uName, MAX_USER_NAME_LENGTH + 1, TEXT("<unknown>"));
             } else {
  #ifndef UNICODE
-                uName = pw->pw_name;
+                _sntprintf(uName, MAX_USER_NAME_LENGTH + 1, TEXT("%s"), pw->pw_name);
  #else
                 req = mbstowcs(NULL, pw->pw_name, MBSTOWCS_QUERY_LENGTH);
                 if (req == (size_t)-1) {
                     return;
                 }
-                uName = malloc(sizeof(TCHAR) * (req + 1));
-                if (!uName) {
-                    outOfMemory(TEXT("DSCS"), 1);
-                    return;
+                if (req > MAX_USER_NAME_LENGTH) {
+                    req = MAX_USER_NAME_LENGTH;
                 }
                 mbstowcs(uName, pw->pw_name, req + 1);
                 uName[req] = TEXT('\0'); /* Avoid bufferflows caused by badly encoded characters. */
@@ -491,9 +499,6 @@ void descSignal(siginfo_t *sigInfo) {
                 TEXT("  signal generated by PID: %d (Session PID: %d), UID: %d (%s)"),
  #endif
                 sigInfo->si_pid, getsid(sigInfo->si_pid), sigInfo->si_uid, uName);
- #ifdef UNICODE
-            free(uName);
- #endif
         }
 #endif
     }
@@ -568,7 +573,7 @@ void sigActionChildDeath(int sigNum, siginfo_t *sigInfo, void *na) {
     /* On UNIX, when a Child process changes state, a SIGCHLD signal is sent to the parent.
      *  The parent should do a wait to make sure the child is cleaned up and doesn't become
      *  a zombie process. */
-
+                
     threadId = pthread_self();
     if (timerThreadSet && pthread_equal(threadId, timerThreadId)) {
         if (wrapperData->isDebugging) {
@@ -1063,6 +1068,17 @@ int wrapperBuildJavaCommand() {
     int length, i;
 
     /* If this is not the first time through, then dispose the old command array */
+    if (wrapperData->jvmVersionCommand) {
+        i = 0;
+        while(wrapperData->jvmVersionCommand[i] != NULL) {
+            free(wrapperData->jvmVersionCommand[i]);
+            wrapperData->jvmVersionCommand[i] = NULL;
+            i++;
+        }
+
+        free(wrapperData->jvmVersionCommand);
+        wrapperData->jvmVersionCommand = NULL;
+    }
     if (wrapperData->jvmCommand) {
         i = 0;
         while(wrapperData->jvmCommand[i] != NULL) {
@@ -1091,28 +1107,37 @@ int wrapperBuildJavaCommand() {
         return TRUE;
     }
 
-    if (wrapperData->commandLogLevel != LEVEL_NONE) {
-        for (i = 0; i < length; i++) {
-            log_printf(WRAPPER_SOURCE_WRAPPER, wrapperData->commandLogLevel,
-                TEXT("Command[%d] : %s"), i, strings[i]);
-        }
-
-        if (wrapperData->environmentClasspath) {
-            log_printf(WRAPPER_SOURCE_WRAPPER, wrapperData->commandLogLevel, TEXT(
-                "Classpath in Environment : %s"), wrapperData->classpath);
-        }
+    /* Allocate memory to hold array of version command strings.  The array is itself NULL terminated */
+    wrapperData->jvmVersionCommand = malloc(sizeof(TCHAR *) * (2 + 1));
+    if (!wrapperData->jvmVersionCommand) {
+        outOfMemory(TEXT("WBJC"), 1);
+        return TRUE;
     }
-
-    if (wrapperData->environmentClasspath) {
-        setEnv(TEXT("CLASSPATH"), wrapperData->classpath, ENV_SOURCE_WRAPPER);
+    memset(wrapperData->jvmVersionCommand, 0, sizeof(TCHAR *) * (2 + 1));
+    /* Java Command */
+    wrapperData->jvmVersionCommand[0] = malloc(sizeof(TCHAR) * (_tcslen(strings[0]) + 1));
+    if (!wrapperData->jvmVersionCommand[0]) {
+        outOfMemory(TEXT("WBJC"), 2);
+        return TRUE;
     }
-
+    _tcsncpy(wrapperData->jvmVersionCommand[0], strings[0], _tcslen(strings[0]) + 1);
+    /* -version */
+    wrapperData->jvmVersionCommand[1] = malloc(sizeof(TCHAR) * (8 + 1));
+    if (!wrapperData->jvmVersionCommand[1]) {
+        outOfMemory(TEXT("WBJC"), 3);
+        return TRUE;
+    }
+    _tcsncpy(wrapperData->jvmVersionCommand[1], TEXT("-version"), 8 + 1);
+    /* NULL */
+    wrapperData->jvmVersionCommand[2] = NULL;
+    
     /* Allocate memory to hold array of command strings.  The array is itself NULL terminated */
     wrapperData->jvmCommand = malloc(sizeof(TCHAR *) * (length + 1));
     if (!wrapperData->jvmCommand) {
         outOfMemory(TEXT("WBJC"), 1);
         return TRUE;
     }
+    memset(wrapperData->jvmCommand, 0, sizeof(TCHAR *) * (length + 1));
     /* number of arguments + 1 for a NULL pointer at the end */
     for (i = 0; i <= length; i++) {
         if (i < length) {
@@ -1122,6 +1147,8 @@ int wrapperBuildJavaCommand() {
                 return TRUE;
             }
             _tcsncpy(wrapperData->jvmCommand[i], strings[i], _tcslen(strings[i]) + 1);
+            
+            wrapperData->jvmCommand[i] = wrapperPostProcessCommandElement(wrapperData->jvmCommand[i]);
         } else {
             wrapperData->jvmCommand[i] = NULL;
         }
@@ -1134,16 +1161,154 @@ int wrapperBuildJavaCommand() {
 }
 
 /**
- * Launches a JVM process and stores it internally.
+ * Calculate the total length of the environment assuming that they are separated by spaces.
  */
-void wrapperExecute() {
+size_t wrapperCalculateEnvironmentLength() {
+    /* The compiler won't let us reverence environ directly in the for loop on OSX because it is actually a function. */
+    char **environment = environ;
+    size_t i;
+    size_t len;
+    size_t lenTotal;
+    
+    i = 0;
+    lenTotal = 0;
+    while (environment[i]) {
+        /* All we need is the length so we don't actually have to convert them. */
+        len = mbstowcs(NULL, environment[i], MBSTOWCS_QUERY_LENGTH);
+        if (len == (size_t)-1) {
+            /* Invalid string.  Skip. */
+        } else {
+            /* Add length of variable + null + pointer to next element */
+            lenTotal += len + 1 + sizeof(char *);
+        }
+        i++;
+    }
+    /* Null termination of the list. */
+    lenTotal += sizeof(char *) + sizeof(char *);
+    
+    return lenTotal;
+}
+
+/**
+ * Create a child process to print the Java version running the command:
+ *    /path/to/java -version
+ *  After printing the java version, the process is terminated.
+ * 
+ * In case the JVM is slow to start, it will time out after
+ * the number of seconds set in "wrapper.java.version.timeout".
+ * 
+ * Note: before the timeout is reached, the user can ctrl+c to stop the Wrapper.
+ */
+void launchChildProcessPrintJavaVersion() {
+    int blockTimeout;      /* max time (in ms) to wait for the child process to terminate */
+    int result;            /* result of waitpid */
+    int status;            /* status of child process */
+    pid_t procJavaVersion; /* pid of the child process */
+    
+    /* Create the child process! */
+    procJavaVersion = fork();
+                
+    if (procJavaVersion >= 0) {
+        /* fork successful */
+        
+        if (procJavaVersion == 0) {
+            /* in the child process */
+            TCHAR *javaVersionArgv[3];
+            javaVersionArgv[0] = wrapperData->jvmCommand[0];
+            javaVersionArgv[1] = TEXT("-version");
+            javaVersionArgv[2] = 0;
+            _texecvp(wrapperData->jvmCommand[0], javaVersionArgv);
+        } else {
+            /* in the parent process */
+            /* Note: on CentOS, in case we don't call waitpid(), then the child process becomes a zombie */
+            /* Note2: use _tprintf to log messages instead of log_printf. With log_printf, the columns are printed twice */
+            
+            /* If the user set the value to 0, then we will wait indefinitely. */
+            blockTimeout = getIntProperty(properties, TEXT("wrapper.java.version.timeout"), DEFAULT_JAVA_VERSION_TIMEOUT) * 1000;
+            
+            if (blockTimeout > 0) {
+                while (((result = waitpid(procJavaVersion, &status, WNOHANG)) == 0) && (blockTimeout > 0)) {
+#ifdef _DEBUG
+                    _tprintf(TEXT("Child process: Java version: waiting... result=%d waitpidStatLoc=%d blockTimeout=%d\n"), result, status, blockTimeout);
+#endif
+                    wrapperSleep(100);
+                    blockTimeout -= 100;
+                }
+            } else {
+                /* Wait indefinately. */
+                result = waitpid(procJavaVersion, &status, 0);
+            }
+            
+            if (result == 0) {
+                /* Timed out. */
+                _tprintf(TEXT("Child process: Java version: timed out\n"));
+                kill(procJavaVersion, SIGKILL);
+            } else if (result > 0) {
+                /* Process completed. */
+#ifdef _DEBUG
+                _tprintf(TEXT("Child process: Java version: successful\n"));
+#endif
+            } else {
+                /* Wait failed. */
+                _tprintf(TEXT("Child process: Java version: wait failed\n"));
+                kill(procJavaVersion, SIGKILL);
+            }
+        }
+    } else {
+        /* Fork failed. */
+        log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT("Unable to spwan process to output Java version: %s"), getLastErrorText());
+    }
+}
+
+/**
+ * Launches a JVM process and stores it internally.
+ *
+ * @return TRUE if there were any problems.  When this happens the Wrapper will not try to restart.
+ */
+int wrapperExecute() {
+    int i;
     pid_t proc;
+    int execErrno;
+    size_t lenCmd;
+    size_t lenEnv;
 
     /* Create the pipe. */
     if (pipe(pipedes) < 0) {
         log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
                    TEXT("Could not init pipe: %s"), getLastErrorText());
-        return;
+        return TRUE;
+    }
+    
+    /* Log the Java commands. */
+    
+    /* If the JVM version printout is requested then log its command line first. */
+    if (wrapperData->printJVMVersion) {
+        if (wrapperData->isDebugging) {
+            log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_DEBUG, TEXT("Java Command Line (Query Java Version):"));
+            for (i = 0; wrapperData->jvmVersionCommand[i] != NULL; i++) {
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_DEBUG,
+                    TEXT("  Command[%d] : %s"), i, wrapperData->jvmVersionCommand[i]);
+            }
+        }
+    }
+    
+    /* Log ghe application java command line */
+    if (wrapperData->commandLogLevel != LEVEL_NONE) {
+        log_printf(WRAPPER_SOURCE_WRAPPER, wrapperData->commandLogLevel, TEXT("Java Command Line:"));
+        for (i = 0; wrapperData->jvmCommand[i] != NULL; i++) {
+            log_printf(WRAPPER_SOURCE_WRAPPER, wrapperData->commandLogLevel,
+                TEXT("  Command[%d] : %s"), i, wrapperData->jvmCommand[i]);
+        }
+
+        if (wrapperData->environmentClasspath) {
+            log_printf(WRAPPER_SOURCE_WRAPPER, wrapperData->commandLogLevel,
+                TEXT("  Classpath in Environment : %s"), wrapperData->classpath);
+        }
+    }
+
+    /* Update the CLASSPATH in the environment if requested so the JVM can access it. */ 
+    if (wrapperData->environmentClasspath) {
+        setEnv(TEXT("CLASSPATH"), wrapperData->classpath, ENV_SOURCE_APPLICATION);
     }
 
     /* Make sure the log file is closed before the Java process is created.  Failure to do
@@ -1169,8 +1334,9 @@ void wrapperExecute() {
         log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_FATAL,
                    TEXT("Could not spawn JVM process: %s"), getLastErrorText());
 
-        /* The pipedes array is global so do not close the pipes. */
 
+        /* The pipedes array is global so do not close the pipes. */
+        return TRUE;
     } else {
         /* Reset the exit code when we launch a new JVM. */
         wrapperData->exitCode = 0;
@@ -1180,6 +1346,7 @@ void wrapperExecute() {
 
         if (proc == 0) {
             /* We are the child side. */
+            
 
             /* Set the umask of the JVM */
             umask(wrapperData->javaUmask);
@@ -1194,14 +1361,18 @@ void wrapperExecute() {
             if (dup2(pipedes[PIPE_WRITE_END], STDOUT_FILENO) < 0) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
                     TEXT("%sUnable to set JVM's stdout: %s"), LOG_FORK_MARKER, getLastErrorText());
-                return;
+                /* This process needs to end. */
+                exit(1);
+                return TRUE; /* Will not get here. */
             }
 
-            /* Send errors to the pipe by dupicating the pipe fd and setting the copy as the stderr fd. */
+            /* Send errors to the pipe by duplicating the pipe fd and setting the copy as the stderr fd. */
             if (dup2(pipedes[PIPE_WRITE_END], STDERR_FILENO) < 0) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
                     TEXT("%sUnable to set JVM's stderr: %s"), LOG_FORK_MARKER, getLastErrorText());
-                return;
+                /* This process needs to end. */
+                exit(1);
+                return TRUE; /* Will not get here. */
             }
             
             /* Close both ends of the pipe as we have already duplicated the Write end for our purposes. */
@@ -1213,23 +1384,32 @@ void wrapperExecute() {
             /* forking at this point, the child process has set all pipes already, so no
                further assignments needed */
             if (wrapperData->printJVMVersion) {
-                if (fork() == 0) {
-                    /* in the child */
-                    TCHAR *javaVersionArgv[3];
-                    javaVersionArgv[0] = wrapperData->jvmCommand[0];
-                    javaVersionArgv[1] = TEXT("-version");
-                    javaVersionArgv[2] = 0;
-                    _texecvp(wrapperData->jvmCommand[0], javaVersionArgv);
-                }
-                /* we don't care about errors here, since they will be reported later */
+                launchChildProcessPrintJavaVersion();
             }
+            
             /* The pipedes array is global so do not close the pipes. */
             /* Child process: execute the JVM. */
             _texecvp(wrapperData->jvmCommand[0], wrapperData->jvmCommand);
+            execErrno = errno;
 
             /* We reached this point...meaning we were unable to start. */
             log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR,
-                TEXT("%sUnable to start JVM: %s (%d)"), LOG_FORK_MARKER, getLastErrorText(), errno);
+                TEXT("%sUnable to start JVM: %s (%d)"), LOG_FORK_MARKER, getLastErrorText(), execErrno);
+            if (execErrno == E2BIG) {
+                /* Command line too long. */
+                /* Calculate the total length of the command line. */
+                lenCmd = 0;
+                for (i = 0; wrapperData->jvmCommand[i] != NULL; i++) {
+                    lenCmd += _tcslen(wrapperData->jvmCommand[i]) + 1;
+                }
+                lenEnv = wrapperCalculateEnvironmentLength();
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT("%s  The generated command line plus the environment was larger than the maximum allowed."), LOG_FORK_MARKER);
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT("%s  The current length is %d bytes of which %d is the command line, and %d is the environment."), LOG_FORK_MARKER, lenCmd + lenEnv + 1, lenCmd, lenEnv); 
+                log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT("%s  It is not possible to calculate an exact maximum length as it depends on a number of factors for each system."), LOG_FORK_MARKER);
+
+                /* TODO: Figure out a way to inform the Wrapper not to restart and try again as repeatedly doing this is meaningless. */
+            }
+
 
             if (wrapperData->isAdviserEnabled) {
                 log_printf(WRAPPER_SOURCE_WRAPPER, LEVEL_ADVICE, TEXT("%s"), LOG_FORK_MARKER );
@@ -1254,8 +1434,9 @@ void wrapperExecute() {
 
             /* This process needs to end. */
             exit(1);
+            return TRUE; /* Will not get here. */
         } else {
-            /* We are the parent side. */
+            /* We are the parent side and need to assume that at this point the JVM is up. */
             wrapperData->javaPID = proc;
             
             /* Close the write end as it is not used. */
@@ -1295,6 +1476,8 @@ void wrapperExecute() {
                         TEXT("Unable to write the Java Id file: %s"), wrapperData->javaIdFilename);
                 }
             }
+            
+            return FALSE;
         }
     }
 }
@@ -1662,7 +1845,7 @@ int setWorkingDir(TCHAR *app) {
     }
 
     /* Set a variable to the location of the binary. */
-    setEnv(TEXT("WRAPPER_BIN_DIR"), szPath, ENV_SOURCE_WRAPPER);
+    setEnv(TEXT("WRAPPER_BIN_DIR"), szPath, ENV_SOURCE_APPLICATION);
 
     if (wrapperSetWorkingDir(szPath, TRUE)) {
         free(szPath);
@@ -1688,7 +1871,7 @@ int main(int argc, char **argv) {
 #ifdef UNICODE
     size_t req;
     TCHAR **argv;
-
+    
 #ifdef FREEBSD
     /* In the case of FreeBSD, we need to dynamically load and initialize the iconv library to work with all versions of FreeBSD. */
     if (loadIconvLibrary()) {
@@ -1696,36 +1879,10 @@ int main(int argc, char **argv) {
         /* Don't call appExit here as we are not far enough along. */
         return 1;
     }
-#endif    
-    
-    /* Create UNICODE versions of the argv array for internal use. */
-    argv = malloc(argc * sizeof *argv );
-    if(!argv) {
-        _tprintf(TEXT("Out of Memory in Main\n"));
-        appExit(1, 0, NULL);
-        return 1;
-    }
-    for (i = 0; i < argc; i++) {
-        req = mbstowcs(NULL, cargv[i], MBSTOWCS_QUERY_LENGTH);
-        if (req == (size_t)-1) {
-            _tprintf(TEXT("Encoding problem with arguments in Main\n"));
-            return 1;
-        }
-        argv[i] = malloc(sizeof(TCHAR) * (req + 1));
-        if (!argv[i]) {
-            _tprintf(TEXT("Out of Memory in Main\n"));
-            while(--i > 0) {
-                free(argv[i]);
-            }
-            free(argv);
-            appExit(1, 0, argv);
-            return 1;
-        }
-        mbstowcs(argv[i], cargv[i], req + 1);
-        argv[i][req] = TEXT('\0'); /* Avoid bufferflows caused by badly encoded characters. */
-    }
-
-#endif
+#endif  
+  
+    /* we should set the locale before trying to convert cargv to argv,
+     * because there might be accentued letter in cargv */
     retLocale = _tsetlocale(LC_ALL, TEXT(""));
     if (retLocale) {
 #if defined(UNICODE)
@@ -1741,6 +1898,38 @@ int main(int argc, char **argv) {
 #endif
         */
     }
+    
+    /* Create UNICODE versions of the argv array for internal use. */
+    argv = malloc(argc * sizeof(TCHAR *));
+    if (!argv) {
+        _tprintf(TEXT("Out of Memory in Main\n"));
+        appExit(1, 0, NULL);
+        return 1;
+    }
+    for (i = 0; i < argc; i++) {
+        req = mbstowcs(NULL, cargv[i], MBSTOWCS_QUERY_LENGTH);
+        if (req == (size_t)-1) {
+            _tprintf(TEXT("Encoding problem with arguments in Main\n"));
+            free(argv);
+            appExit(1, 0, NULL);
+            return 1;
+        }
+        argv[i] = malloc(sizeof(TCHAR) * (req + 1));
+        if (!argv[i]) {
+            _tprintf(TEXT("Out of Memory in Main\n"));
+            while (--i > 0) {
+                free(argv[i]);
+            }
+            free(argv);
+            appExit(1, 0, NULL);
+            return 1;
+        }
+        mbstowcs(argv[i], cargv[i], req + 1);
+        argv[i][req] = TEXT('\0'); /* Avoid bufferflows caused by badly encoded characters. */
+    }
+
+#endif
+
     if (wrapperInitialize()) {
         appExit(1, argc, argv);
         return 1; /* For compiler. */
@@ -1799,6 +1988,12 @@ int main(int argc, char **argv) {
     }
 
     /* Load the properties. */
+    /* To make the WRAPPER_LANG references in the configuration work correctly,
+     *  it is necessary to load the configuration twice.
+     * The first time, we want to ignore the return value.  Any errors will be
+     *  suppressed and will get reported again the second time through. */
+    /* From version 3.5.27, the community edition will also preload the configuration properties. */ 
+    wrapperLoadConfigurationProperties(TRUE);
     if (wrapperLoadConfigurationProperties(FALSE)) {
         /* Unable to load the configuration.  Any errors will have already
          *  been reported. */
